# Movable Type plugin for filtering duplicate trackback ping.
#
# Release 2.00 (Oct 05, 2005)
#
# This software is provided as-is. You may use it for commercial or 
# personal use. If you distribute it, please keep this notice intact.
#
# Copyright (c) 2005 Yujiro Araki

package MT::Plugin::duplicateTBPingFilter;
use strict;
use base qw(MT::Plugin);
use vars qw($VERSION);

use MT::TBPing;
use MT::Trackback;

# Set how to filter the title.
# 0:skip / 1:check
our $title = 1;

# Set how to filter the excerpt.
# 0:skip / 1:check
our $excerpt = 1;

# Set the checked number of track backs.
our $counter = 0;

sub BEGIN {
  $VERSION = '2.00';
  my $plugin = MT::Plugin::duplicateTBPingFilter->new ({
      name => 'Duplicate Trackback Ping Filter Plugin',
      description => 'Filter duplicate trackback ping to the same entry.',
      doc_link => 'http://www.koikikukan.com/archives/2005/05/28-235959.php',
      version => $VERSION,
      blog_config_template => \&template,
      settings => new MT::PluginSettings ([
      ['duplicate_TBPing_Filter', { Default => '0', Scope => 'blog' }],
      ]),
      });
  MT->add_plugin ($plugin);
  MT->add_callback ( 'TBPingFilter', 11, $plugin, sub { duplicate_tbping_filter($plugin, @_)} );
}

sub is_same_url {
   my ($new_url, $old_url) = @_;
   return ($new_url eq $old_url);
}

sub is_same_title {
    my ($new_title, $old_title) = @_;
    if (!$title) {
        return 1;
    } else {
        return ($new_title eq $old_title);
    }
}

sub is_same_excerpt {
    my ($new_excerpt, $old_excerpt) = @_;
    if (!$excerpt) {
        return 1;
    } else {
        return ($new_excerpt eq $old_excerpt);
    }
}

sub duplicate_tbping_filter {
    my $plugin = shift;
    my ($eh, $app, $ping) = @_;
    my $blog = $ping->blog;
    unless ($plugin->get_config_value('duplicate_TBPing_Filter', 'blog:'.$blog->id)) {
        return 1;
    }
    my $ping_iter = MT::TBPing->load_iter({tb_id=>$ping->tb_id},
                                          {'sort' => 'created_on',direction => 'descend'});
    while (my $old_ping = $ping_iter->()) {
         if ($ping->id ne $old_ping->id) {
             if (&is_same_url($ping->source_url, $old_ping->source_url) &&
                 &is_same_title($ping->title, $old_ping->title) &&
                 &is_same_excerpt($ping->excerpt, $old_ping->excerpt)) {
                 $app->log("Filter duplicate trackback ping from the following URL:" . $ping->source_url);
                 return 0;
             }
         }
         if ($counter) {
             $counter--;
             if (!$counter) {
                 return 1;
             }
         }
    }
    return 1;
}

sub template {
    return <<EOF;
<div class="setting">
  <div class="label">
    <label for="duplicate_TBPing_Filter">Configuration:</label>
  </div>
  <div class="field">
    <p><input type="checkbox" class="cb" name="duplicate_TBPing_Filter" id="duplicate_TBPing_Filter" <TMPL_IF NAME="DUPLICATE_TBPING_FILTER_1">checked</TMPL_IF> value="1"/> Filter duplicate trackback ping to the same entry.</p>
  </div>
</div>
EOF
}

1;
